
/*  $Header: /cvs/faces/faces/compface/cmain.c,v 1.3 2002/02/21 15:42:54 devface Exp $
 *
 *  Compface - 48x48x1 image compression.
 *
 *  Copyright (c) 1990-2002 James Ashton - Sydney University
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged. 
 *
 *  No responsibility is taken for any errors on inaccuracies inherent
 *  either to the comments or the code of this program, but if reported
 *  to me, then an attempt will be made to fix them.
 */

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "compface.h"

#define  STRCAT        (void) strcat
#define  STRCPY        (void) strcpy
#define  WRITE         (void) write

/* The buffer is longer than needed to handle sparse input formats */

#define FACEBUFLEN 2048
char fbuf[FACEBUFLEN];

/* IO file descriptors and their names */

int infile    = 0;
char *inname  = "<stdin>";
int outfile   = 1;
char *outname = "<stdout>";

char *cmdname;            /* Basename of executable */

/* Error handling definitions follow */

#define ERR         strerror(errno)
#define INITERR(s)  { \
                        STRCPY(fbuf, cmdname); \
                        STRCAT(fbuf, ": "); \
                        STRCAT(fbuf, (s));\
                    }
#define ADDERR(s)   STRCAT(fbuf, (s));
#define ERROR       { \
                        STRCAT(fbuf, "\n"); \
                        WRITE(2, fbuf, strlen(fbuf)); \
                        exit(1); \
                    }
#define INITWARN(s) { \
                        STRCPY(fbuf, cmdname); \
                        STRCAT(fbuf, ": (warning) "); \
                        STRCAT(fbuf, (s)); \
                    }
#define ADDWARN(s)  STRCAT(fbuf, (s));
#define WARN        { \
                        STRCAT(fbuf, "\n"); \
                        WRITE(2, fbuf, strlen(fbuf)); \
                    }

int
main(int argc, char *argv[])
{
    cmdname = *argv;
    while (**argv) {
        if (*((*argv)++) == '/') {
            cmdname = *argv;               /* Find the command's basename */
        }
    }

    if (argc > 3) {
        INITERR("usage: ")
        ADDERR(cmdname)
        ADDERR(" [infile [outfile]]")
        ERROR
    }

    if ((argc > 1) && strcmp(*++argv, "-")) {
        inname = *argv;
        if ((infile = open(inname, O_RDONLY)) == -1) {
            INITERR(inname)
            ADDERR(": ")
            ADDERR(ERR)
            ERROR
        }
    }

    if (argc > 2) {
        outname = *++argv;
        if ((outfile = open(outname, O_WRONLY | O_CREAT | 
                                     O_TRUNC, 0644)) == -1) {
            INITERR(outname)
            ADDERR(": ")
            ADDERR(ERR)
            ERROR
        }
    }

    (void) ReadBuf();
    switch (compface(fbuf)) {
        case -2 : INITERR("internal error")
                  ERROR
        case -1 : INITERR(inname)
                  ADDERR(": insufficient or invalid data")
                  ERROR
        case  1 : INITWARN(inname)
                  ADDWARN(": excess data ignored")
                  WARN
        default : ;
    }
    (void) WriteBuf();
    exit(0);
/*NOTREACHED*/
}


int
WriteBuf()
{
    char *s, *t;
    int len;

    s = fbuf;
    t = s + strlen(s);
    while (s < t) {
        if ((len = write(outfile, s, t - s)) == -1) {
            INITERR(outname)
            ADDERR(": ")
            ADDERR(ERR)
            ERROR
        }
        s += len;
    }
    return(0);
}


int
ReadBuf()
{
    int count, len;
    char *t;

    count = 0;
    t = fbuf;
    while ((len = read(infile, t, FACEBUFLEN - count))) {
        if (len == -1) {
            INITERR(inname)
            ADDERR(": ")
            ADDERR(ERR)
            ERROR
        }
        t += len;
        if ((count += len) >= FACEBUFLEN) {
            INITWARN(inname)
            ADDWARN(" exceeds internal buffer size.  Data may be lost")
            WARN
            break;
        }
    }
    *t = '\0';
    return(count);
}
